import logging
import os
from pathlib import Path
from os import path
from shutil import which

import yamale
from yamale import YamaleError
from yaml.scanner import ScannerError

from rsync_tasks import exceptions
from rsync_tasks.exceptions import BackupTaskError, TaskFailed, LocalPathNotWritable, TargetFailed

_LOGGER = logging.getLogger(__package__)


class RsyncTasks:
    def __init__(self,
                 tasks: list[dict[str, str]],
                 flags: str,
                 base_path_source: str,
                 base_path_destination: str,
                 **kwargs):
        self._tasks = tasks

        self._flags = flags
        self._base_path_source = Path(base_path_source)
        self._base_path_destination = Path(base_path_destination)

        # Checks if the necessary system tools are available
        RsyncTasks._check_tools()

    @classmethod
    def make_from_config(cls, config_path: str) -> RsyncTasks:
        # Loads the config schema to validate the config
        schema = yamale.make_schema(path.join(path.dirname(__file__), "resources/config.schema.yaml"))

        # Tries to load config file
        try:
            config = yamale.make_data(config_path)
        except FileNotFoundError:
            _LOGGER.error("Configuration file '{}' not found".format(config_path))
            raise exceptions.ConfigNotFound(config_path)
        except ScannerError as e:
            _LOGGER.error("Invalid configuration file '{}'\n{}".format(config_path, e))
            raise exceptions.InvalidConfig(e)

        # Tries to validate the configuration with the schema
        try:
            yamale.validate(schema, config)
        except YamaleError as e:
            _LOGGER.error("Invalid configuration file '{}'\n{}".format(config_path, e))
            raise exceptions.InvalidConfig(e)

        _LOGGER.info("Configuration loaded")

        # create instance form config
        config, _ = config[0]
        return cls(**config)

    @staticmethod
    def _check_tools() -> None:
        # Checks if 'rsync' is available
        if which("rsync") is None:
            e = exceptions.RsyncNotAvailable
            _LOGGER.error(e)
            raise e

        _LOGGER.info("All the needed tools are available")

    def start(self):
        for task in self._tasks:
            try:
                source = self._base_path_source / task["s"]
                destination = self._base_path_destination / task["d"]

                # checks if the local path is writable
                #if not os.access(destination, os.W_OK):
                #    raise LocalPathNotWritable(str(destination))

                # creates the command for the current target
                command = self._mk_command(source, destination)

                _LOGGER.debug("Run command `{}`".format(command))

                # launches the command
                if os.system(command) != 0:
                    _LOGGER.warning("Failed target `{}`".format(command))
                    raise TargetFailed(task["s"])

            except BackupTaskError as e:
                _LOGGER.error("Task {} failed\n{}".format(None, e))

                raise TaskFailed(e)

    def _mk_command(self, source: Path, destination) -> str:
        command = ["rsync"]

        # adds necessary flag
        command += [self._flags]

        # adds the paths
        command += [str(source), str(destination)]

        return " ".join(command)



class RsyncTasksError(Exception):
    pass


class RsyncNotAvailable(RsyncTasksError):
    def __str__(self) -> str:
        return "'rsync' tool not found"


class ConfigError(RsyncTasksError):
    pass


class ConfigNotFound(ConfigError):
    def __init__(self, file: str):
        self._file = file

    def __str__(self) -> str:
        return "Configuration file '{}' not found".format(self._file)


class InvalidConfig(ConfigError):
    pass


class BackupTaskError(RsyncTasksError):
    pass


class LocalPathNotWritable(BackupTaskError):
    def __init__(self, path: str):
        self._path = path

    def __str__(self) -> str:
        return "Directory '{}' is not writable".format(self._path)


class IdentityFileUnreadable(BackupTaskError):
    def __init__(self, file: str):
        self._file = file

    def __str__(self) -> str:
        return "Identity file '{}' is not readable".format(self._file)


class TargetFailed(BackupTaskError):
    def __init__(self, path: str):
        self._path = path

    def __str__(self) -> str:
        return "Failed sync of '{}'".format(self._path)


class TaskFailed(BackupTaskError):
    pass



#!/usr/bin/env python3
import argparse
import logging
import sys

from rsync_tasks import RsyncTasks, __version__
from rsync_tasks.exceptions import ConfigError
from .log import logger_setup

_LOGGER = logging.getLogger(__package__)


def main() -> int:
    # gets inline arguments
    parser = argparse.ArgumentParser(prog="rsync_tasks")

    parser.add_argument("-c", "--config", dest="config_path", default="/etc/rsync_tasks/config.yaml",
                        help="configuration file path")
    parser.add_argument("-v", dest="log_level", action="count", default=0,
                        help="number of -v defines level of verbosity")

    parser.add_argument("--version", action="version", version="rsync_tasks {}".format(__version__))

    # parses args
    args = vars(parser.parse_args())

    # sets verbosity level
    logger_setup(args["log_level"])

    # removes no longer necessary log level from args
    del args["log_level"]

    # creates an instance of RsyncTasks
    try:
        rt = RsyncTasks.make_from_config(**args)
    except ConfigError as e:
        _LOGGER.critical("Configuration issue: I give up")
        return 1

    _LOGGER.info("RsyncTasks is ready to start")

    # starts backup operations
    rt.start()

    return 0


if __name__ == "__main__":
    sys.exit(main())



import importlib
import inspect
import logging
import pkgutil
from dataclasses import dataclass
from types import ModuleType
from typing import Union


@dataclass
class LogLevel:
    module: int
    sub_modules: int
    other_modules: int


LOG_LEVELS = [
    LogLevel(logging.INFO, logging.CRITICAL, logging.CRITICAL),
    LogLevel(logging.INFO, logging.ERROR, logging.CRITICAL),
    LogLevel(logging.INFO, logging.WARNING, logging.CRITICAL),
    LogLevel(logging.INFO, logging.INFO, logging.CRITICAL),
    LogLevel(logging.INFO, logging.INFO, logging.WARNING),
    LogLevel(logging.DEBUG, logging.DEBUG, logging.WARNING)
]
_LOGGER = logging.getLogger(__package__)

_log_level: Union[LogLevel, None] = None


def get_log_level() -> Union[LogLevel, None]:
    return _log_level


def logger_setup(log_level: Union[int, LogLevel], base_module: ModuleType = None):
    # if the log level is in int format, converts it in the associated LogLevel
    if isinstance(log_level, int):
        try:
            log_level = LOG_LEVELS[log_level]
        except IndexError:
            log_level = LOG_LEVELS[-1]

    # sets format for the log
    logging.basicConfig(format="%(levelname)s|%(name)s>%(message)s", level=log_level.other_modules)

    # if the base module is not provided retrieves this form the stack
    if base_module is None:
        frm = inspect.stack()[1]
        mod = inspect.getmodule(frm[0])
        base_module = importlib.import_module(mod.__package__)

    # sets log level for the packet
    logging.getLogger(base_module.__package__).setLevel(log_level.module)

    # sets log level for local packages
    for _, modname, _ in pkgutil.walk_packages(path=base_module.__path__,
                                               prefix=base_module.__name__ + ".",
                                               onerror=lambda x: None):
        logging.getLogger(modname).setLevel(log_level.sub_modules)

    _LOGGER.info("Log levels set as {}".format(log_level))

    # saves the log level
    global _log_level
    _log_level = log_level



from os import path

from setuptools import setup, find_packages

__version__ = None
exec(open(path.join(path.dirname(__file__), "./rsync_tasks/version.py")).read())
if __version__ == "dev":
    __version__ = "0.0"

with open(path.join(path.dirname(__file__), "README.md")) as f:
    _LONG_DESCRIPTION = f.read()

setup(
    name="rsync-tasks",
    packages=find_packages(),
    version=__version__,
    license="gpl-3.0",
    description="A simple configurable wrapper around rsync",
    #long_description=_LONG_DESCRIPTION,
    #long_description_content_type="text/markdown",
    author="Roberto Bochet",
    author_email="robertobochet@gmail.com",
    url="https://github.com/RobertoBochet/rsync-tasks",
    keywords=["rsync", "wrapper", "tool"],
    install_requires=[
        "yamale~=3.0.4",
        "PyYAML~=5.3.1"
    ],
    entry_points={
        "console_scripts": [
            "rsync-tasks = rsync_tasks.__main__:main"
        ]
    },
    package_data={
        "rsync-tasks": ["resources/config.schema.yaml"]
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: System Administrators",
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3"
    ],
    python_requires=">=3.8"
)
